unit ESImage;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;
type
 TESimage=class(TGraphicControl)
private
   FPicture1:TPicture;
   FPicture2:TPicture;
   FStretch :boolean;
   FAutosize:boolean;
   FCenter  :boolean;
   Dest: TRect;
   mousein:boolean;
 Function GetCanvas1:TCanvas;
 Function GetCanvas2:TCanvas;
 Procedure PictureChanged(Sender:TObject);
 procedure SetAutoSize(Value: Boolean);
 procedure SetCenter(Value: Boolean);
 procedure SetPicture1(Value: TPicture);
 procedure SetPicture2(Value: TPicture);
 procedure SetStretch(Value: Boolean);
protected
 Procedure WMMouseMove(var msg: TWMMouseMove);Message WM_MOUSEMOVE;
 function GetPalette: HPALETTE; override;
 procedure Paint; override;
public
 constructor Create(AOwner: TComponent); override;
 destructor Destroy; override;
 property Canvas1: TCanvas read GetCanvas1;
 property Canvas2: TCanvas read GetCanvas2;
published
 property Align;
 property AutoSize: Boolean read FAutoSize write SetAutoSize default False;
 property Center: Boolean read FCenter write SetCenter default False;
 property DragCursor;
 property DragMode;
 property Enabled;
 property ParentShowHint;
 property Picture1: TPicture read FPicture1 write SetPicture1;
 property Picture2: TPicture read FPicture2 write SetPicture2;
 property PopupMenu;
 property ShowHint;
 property Stretch: Boolean read FStretch write SetStretch default False;
 property Visible;
 property OnClick;
 property OnDblClick;
 property OnDragDrop;
 property OnDragOver;
 property OnEndDrag;
 property OnMouseDown;
 property OnMouseMove;
 property OnMouseUp;
 property OnStartDrag;
end;

procedure register;

implementation
constructor TESimage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csReplicatable];
  FPicture1 := TPicture.Create;
  FPicture2 := TPicture.Create;
  FPicture1.OnChange := PictureChanged;
  FPicture2.OnChange := PictureChanged;
  Height := 105;
  Width := 105;
end;

destructor TESimage.Destroy;
begin
  FPicture1.Free;
  FPicture2.Free;
  inherited Destroy;
end;

function TESimage.GetPalette: HPALETTE;
begin
  Result := 0;
  if FPicture1.Graphic is TBitmap then
    Result := TBitmap(FPicture1.Graphic).Palette;
  if FPicture2.Graphic is TBitmap then
    Result := TBitmap(FPicture2.Graphic).Palette;
end;

procedure TESimage.Paint;
begin
  if csDesigning in ComponentState then
    with inherited Canvas do
    begin
      Pen.Style := psDash;
      Brush.Style := bsClear;
      Rectangle(0, 0, Width, Height);
    end;
  if Stretch then
    Dest := ClientRect
  else if Center then
    Dest := Bounds((Width - Picture1.Width) div 2, (Height - Picture1.Height) div 2,
      Picture1.Width, Picture1.Height)
  else
    Dest := Rect(0, 0, Picture1.Width, Picture1.Height);
  with inherited Canvas do
   StretchDraw(Dest, Picture1.Graphic);
end;

Procedure TESimage.WMMouseMove(Var msg: TWMMouseMove);
 Begin
  If not MouseCapture Then
   Begin
    SetCaptureControl(Self);
     If Enabled and Visible and getParentForm(Self).Active Then
       with inherited Canvas do
        StretchDraw(Dest, Picture1.Graphic);
    End;
   If MouseCapture and ((msg.xpos < 0) or (msg.ypos < 0) or (msg.xPos >
      Width) or (msg.Ypos > Height)) and mousein Then
       Begin
        MouseIn := False;
        with inherited Canvas do
         StretchDraw(Dest, Picture1.Graphic);
        SetCaptureControl(Nil);
       End;
   If MouseCapture and ((msg.xpos >= 0) and (msg.ypos >= 0) and (msg.xPos <=
      Width) and (msg.Ypos <= Height)) and not MouseIn Then
       Begin
        MouseIn := True;
        {if not FDown and FEnabled Then begin
        end;}
       with inherited Canvas do
        StretchDraw(Dest, Picture2.Graphic);
       End;
 End;

function TESimage.GetCanvas1: TCanvas;
var
  Bitmap: TBitmap;
begin
  if Picture1.Graphic = nil then
  begin
    Bitmap := TBitmap.Create;
    try
      Bitmap.Width := Width;
      Bitmap.Height := Height;
      Picture1.Graphic := Bitmap;
    finally
      Bitmap.Free;
    end;
  end;
  if Picture1.Graphic is TBitmap then
    Result := TBitmap(Picture1.Graphic).Canvas
  else
    raise EInvalidOperation.Create('Image Canvas Need Bitmap');
end;

function TESimage.GetCanvas2: TCanvas;
var
  Bitmap: TBitmap;
begin
  if Picture2.Graphic = nil then
  begin
    Bitmap := TBitmap.Create;
    try
      Bitmap.Width := Width;
      Bitmap.Height := Height;
      Picture2.Graphic := Bitmap;
    finally
      Bitmap.Free;
    end;
  end;
  if Picture2.Graphic is TBitmap then
    Result := TBitmap(Picture2.Graphic).Canvas
  else
    raise EInvalidOperation.Create('Image Canvas Need Bitmap');
end;

procedure TESimage.SetAutoSize(Value: Boolean);
begin
  FAutoSize := Value;
  PictureChanged(Self);
end;

procedure TESimage.SetCenter(Value: Boolean);
begin
  if FCenter <> Value then
  begin
    FCenter := Value;
    PictureChanged(Self);
  end;
end;

procedure TESimage.SetPicture1(Value: TPicture);
begin
  FPicture1.Assign(Value);
end;

procedure TESimage.SetPicture2(Value: TPicture);
begin
  FPicture2.Assign(Value);
end;

procedure TESimage.SetStretch(Value: Boolean);
begin
  if Value <> FStretch then
  begin
    FStretch := Value;
    PictureChanged(Self);
  end;
end;

procedure TESimage.PictureChanged(Sender: TObject);
begin
  if AutoSize and (Picture1.Width > 0) and (Picture1.Height > 0) then
    SetBounds(Left, Top, Picture1.Width, Picture1.Height);
  if (Picture1.Graphic is TBitmap) and (Picture1.Width >= Width) and
    (Picture1.Height >= Height) then
    ControlStyle := ControlStyle + [csOpaque] else
    ControlStyle := ControlStyle - [csOpaque];
  Invalidate;
end;

procedure Register;
begin
  RegisterComponents('Additional', [TESimage]);
end;


end.unit ESImage;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;
type
 TESimage=class(TGraphicControl)
private
   FPicture1:TPicture;
   FPicture2:TPicture;
   FStretch :boolean;
   FAutosize:boolean;
   FCenter  :boolean;
   Dest: TRect;
   mousein:boolean;
 Function GetCanvas1:TCanvas;
 Function GetCanvas2:TCanvas;
 Procedure PictureChanged(Sender:TObject);
 procedure SetAutoSize(Value: Boolean);
 procedure SetCenter(Value: Boolean);
 procedure SetPicture1(Value: TPicture);
 procedure SetPicture2(Value: TPicture);
 procedure SetStretch(Value: Boolean);
protected
 Procedure WMMouseMove(var msg: TWMMouseMove);Message WM_MOUSEMOVE;
 function GetPalette: HPALETTE; override;
 procedure Paint; override;
public
 constructor Create(AOwner: TComponent); override;
 destructor Destroy; override;
 property Canvas1: TCanvas read GetCanvas1;
 property Canvas2: TCanvas read GetCanvas2;
published
 property Align;
 property AutoSize: Boolean read FAutoSize write SetAutoSize default False;
 property Center: Boolean read FCenter write SetCenter default False;
 property DragCursor;
 property DragMode;
 property Enabled;
 property ParentShowHint;
 property Picture1: TPicture read FPicture1 write SetPicture1;
 property Picture2: TPicture read FPicture2 write SetPicture2;
 property PopupMenu;
 property ShowHint;
 property Stretch: Boolean read FStretch write SetStretch default False;
 property Visible;
 property OnClick;
 property OnDblClick;
 property OnDragDrop;
 property OnDragOver;
 property OnEndDrag;
 property OnMouseDown;
 property OnMouseMove;
 property OnMouseUp;
 property OnStartDrag;
end;

procedure register;

implementation
constructor TESimage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csReplicatable];
  FPicture1 := TPicture.Create;
  FPicture2 := TPicture.Create;
  FPicture1.OnChange := PictureChanged;
  FPicture2.OnChange := PictureChanged;
  Height := 105;
  Width := 105;
end;

destructor TESimage.Destroy;
begin
  FPicture1.Free;
  FPicture2.Free;
  inherited Destroy;
end;

function TESimage.GetPalette: HPALETTE;
begin
  Result := 0;
  if FPicture1.Graphic is TBitmap then
    Result := TBitmap(FPicture1.Graphic).Palette;
  if FPicture2.Graphic is TBitmap then
    Result := TBitmap(FPicture2.Graphic).Palette;
end;

procedure TESimage.Paint;
begin
  if csDesigning in ComponentState then
    with inherited Canvas do
    begin
      Pen.Style := psDash;
      Brush.Style := bsClear;
      Rectangle(0, 0, Width, Height);
    end;
  if Stretch then
    Dest := ClientRect
  else if Center then
    Dest := Bounds((Width - Picture1.Width) div 2, (Height - Picture1.Height) div 2,
      Picture1.Width, Picture1.Height)
  else
    Dest := Rect(0, 0, Picture1.Width, Picture1.Height);
  with inherited Canvas do
   StretchDraw(Dest, Picture1.Graphic);
end;

Procedure TESimage.WMMouseMove(Var msg: TWMMouseMove);
 Begin
  If not MouseCapture Then
   Begin
    SetCaptureControl(Self);
     If Enabled and Visible and getParentForm(Self).Active Then
       with inherited Canvas do
        StretchDraw(Dest, Picture1.Graphic);
    End;
   If MouseCapture and ((msg.xpos < 0) or (msg.ypos < 0) or (msg.xPos >
      Width) or (msg.Ypos > Height)) and mousein Then
       Begin
        MouseIn := False;
        with inherited Canvas do
         StretchDraw(Dest, Picture1.Graphic);
        SetCaptureControl(Nil);
       End;
   If MouseCapture and ((msg.xpos >= 0) and (msg.ypos >= 0) and (msg.xPos <=
      Width) and (msg.Ypos <= Height)) and not MouseIn Then
       Begin
        MouseIn := True;
        {if not FDown and FEnabled Then begin
        end;}
       with inherited Canvas do
        StretchDraw(Dest, Picture2.Graphic);
       End;
 End;

function TESimage.GetCanvas1: TCanvas;
var
  Bitmap: TBitmap;
begin
  if Picture1.Graphic = nil then
  begin
    Bitmap := TBitmap.Create;
    try
      Bitmap.Width := Width;
      Bitmap.Height := Height;
      Picture1.Graphic := Bitmap;
    finally
      Bitmap.Free;
    end;
  end;
  if Picture1.Graphic is TBitmap then
    Result := TBitmap(Picture1.Graphic).Canvas
  else
    raise EInvalidOperation.Create('Image Canvas Need Bitmap');
end;

function TESimage.GetCanvas2: TCanvas;
var
  Bitmap: TBitmap;
begin
  if Picture2.Graphic = nil then
  begin
    Bitmap := TBitmap.Create;
    try
      Bitmap.Width := Width;
      Bitmap.Height := Height;
      Picture2.Graphic := Bitmap;
    finally
      Bitmap.Free;
    end;
  end;
  if Picture2.Graphic is TBitmap then
    Result := TBitmap(Picture2.Graphic).Canvas
  else
    raise EInvalidOperation.Create('Image Canvas Need Bitmap');
end;

procedure TESimage.SetAutoSize(Value: Boolean);
begin
  FAutoSize := Value;
  PictureChanged(Self);
end;

procedure TESimage.SetCenter(Value: Boolean);
begin
  if FCenter <> Value then
  begin
    FCenter := Value;
    PictureChanged(Self);
  end;
end;

procedure TESimage.SetPicture1(Value: TPicture);
begin
  FPicture1.Assign(Value);
end;

procedure TESimage.SetPicture2(Value: TPicture);
begin
  FPicture2.Assign(Value);
end;

procedure TESimage.SetStretch(Value: Boolean);
begin
  if Value <> FStretch then
  begin
    FStretch := Value;
    PictureChanged(Self);
  end;
end;

procedure TESimage.PictureChanged(Sender: TObject);
begin
  if AutoSize and (Picture1.Width > 0) and (Picture1.Height > 0) then
    SetBounds(Left, Top, Picture1.Width, Picture1.Height);
  if (Picture1.Graphic is TBitmap) and (Picture1.Width >= Width) and
    (Picture1.Height >= Height) then
    ControlStyle := ControlStyle + [csOpaque] else
    ControlStyle := ControlStyle - [csOpaque];
  Invalidate;
end;

procedure Register;
begin
  RegisterComponents('Additional', [TESimage]);
end;


end.
